/*
 * Copyright 2013 C&D Elettronica. All rights reserved.
 *
 * This file is part of the FlashMat project.
 */

#include "Font.h"
#include <string.h>

namespace FlashMat {

int8_t Font::x0(uint8_t c) const {
	if (!charExists(c)) {
		return 0;
	}

	return (int8_t) pgm_read_byte(&_chars[chr(c,0)]);
}

uint8_t Font::width(uint8_t c) const {
	if (!charExists(c)) {
		return 0;
	}

	return pgm_read_byte(&_chars[chr(c,1)]);
}

void Font::getMask(uint8_t c, uint8_t mask[/*maskBytesPerChar()*/]) const {
	if (!charExists(c)) {
		memset(mask, 0, bytesPerChar());
		return;
	}

	// set pointer to the first row of the mask (skip x0 and width)
	const uint8_t* b /*PROGMEM*/ = &_chars[chr(c, 2)];
    
	// read the bytes of the mask
	for (uint8_t n = _maskBytesPerChar + 1; --n > 0;) {
		*(mask++) = pgm_read_byte(b++);
	}
}

Font::Font(const uint8_t* chars/*PROGMEM*/) :
		_chars(chars) {

	_y0         = (int8_t) pgm_read_byte(&_chars[0]);
	_height     = pgm_read_byte(&_chars[1]);
	_monoHeight = pgm_read_byte(&_chars[2]);
	_monoWidth  = pgm_read_byte(&_chars[3]);
	_minChar    = pgm_read_byte(&_chars[4]);
	_maxChar    = pgm_read_byte(&_chars[5]);
	_maskHeight = pgm_read_byte(&_chars[6]);
	_maskWidth  = pgm_read_byte(&_chars[7]);

    _maskBytesPerChar = _maskHeight * _maskWidth * 8;
}
}
