/*
 * Copyright 2013 C&D Elettronica. All rights reserved.
 *
 * This file is part of the FlashMat project.
 */

#ifndef FONTS_H_
#define FONTS_H_

#include <inttypes.h>
#include "Font.h"

namespace FlashMat {
    
class Fonts {
public:
   
    enum FontId {
        FM_8x8,
        FM_BIG_8x8,
        FM_5x8,
        
        ARIAL_8x8,
        ARIAL_BLACK_8x8,
        ARIAL_ITALIC_8x8,
        
        COURIER_7x8,
        LUCIDA_7x8,

        TIMES_8x8,

        COMIC_8x8,

        ARIAL_16x16,

        SYM_8x8,
        SYM_16x16,
        
        SERINF_16x16,
        // EXTEND: add your font's name here
        
        NONE, // No characters at all
        _TOTAL_FONTS // keep this as the last entry, to know the number of defined fonts (this is NOT an actual font)
        };

	// Return a font given its id.
	// If it doesn't exist, return either null (if 'nullIfNotAvailable') or the default font (#0).
	// Mind that the default one might be disabled too; in this case the NONE one is returned.
	// If this is null, too, null is returned.
	// To know if a font is available, just use the test:  (getFont(id, true) == 0)
	static const Font* getFont(FontId id, bool nullIfNotAvailable = false);

private:
    static const uint8_t N; //num fonts
	Fonts(); // unused
	Fonts(const Fonts&); //unused
	Fonts& operator=(const Fonts&); // unused
	~Fonts();
};

////////////////////////////////////////////////////////////////
// Must declare here the existing fonts
extern Font font_none;
extern Font font_fm_8x8_ext;
extern Font font_fm_8x8;
extern Font font_fm_big_8x8_ext;
extern Font font_fm_big_8x8;
extern Font font_fm_5x8_ext;
extern Font font_fm_5x8;

extern Font font_arial_8x8_ext;
extern Font font_arial_8x8;
extern Font font_arial_black_8x8_ext;
extern Font font_arial_black_8x8;
extern Font font_arial_italic_8x8_ext;
extern Font font_arial_italic_8x8;

extern Font font_courier_7x8_ext;
extern Font font_courier_7x8;
extern Font font_lucida_7x8_ext;
extern Font font_lucida_7x8;

extern Font font_times_8x8_ext;
extern Font font_times_8x8;

extern Font font_comic_8x8_ext;
extern Font font_comic_8x8;

extern Font font_arial_16x16_ext;
extern Font font_arial_16x16;

extern Font font_sym_8x8_ext;
extern Font font_sym_16x16_ext;

extern Font font_serinf_16x16;
extern Font font_serinf_16x16_ext;
// EXTEND: add your font's declaration here

inline
const Font* Fonts::getFont(Fonts::FontId id, bool nullIfNotAvailable) {
	// List of available animations
	static const Font* fonts[(uint8_t)Fonts::_TOTAL_FONTS] =
        {   // use null in place of the fonts you'd like to disable
            // and mind the & in front of the font's name
            // NOTE: they must be in the same order as in the enum above.
            // If a font has variants (plain, "ext", ...), you must choose one of them
            &font_fm_8x8, // e.g. this can be 'font_fm_8x8' or 'font_fm_8x8_ext' (or 0)
            0, //&font_fm_big_8x8,
            0, //&font_fm_5x8,
                                
            0, //&font_arial_8x8,
            0, //&font_arial_black_8x8,
            0, //&font_arial_italic_8x8,
                                
            0, //&font_courier_7x8,
            0, //&font_lucida_7x8,
                                
            0, //&font_times_8x8,
                                
            0, //&font_comic_8x8,

            &font_arial_16x16,
                                
            0, //&font_sym_8x8_ext,
            0, //&font_sym_16x16_ext,
            
            0, //&font_serinf_16x16,
            
            0, //&font_none, // nothing at all
                                                
            // EXTEND: add here your font's name
        };
	// end of list
    
    const Font* defFont = nullIfNotAvailable ?
                                               0
                                             : (fonts[0] ?
                                                           fonts[0]
                                                         : fonts[(uint8_t)NONE]);
    
    const Font* f = (id >= _TOTAL_FONTS) ?
                                                   defFont
                                                 : fonts[(uint8_t)id];
    
    return (f ? f : defFont);
}

} // namespace

#endif /* FONTS_H_ */
