/*
 * Copyright 2013 C&D Elettronica. All rights reserved.
 *
 * This file is part of the FlashMat project.
 */

#ifndef POINT_H_
#define POINT_H_

#include <inttypes.h>

namespace FlashMat {

typedef int16_t coord_t;

/*
 * Data structure to hold a pair of coordinates. They are of a type wide enough to
 * cover all plausible points on the matrix (and far beyond, to move things around).
 * They can also be negative, to allow positioning to the left/top of the matrix.
 */
struct Point {
    coord_t x;
    coord_t y;
    
    explicit Point(coord_t x = 0, coord_t y = 0): x(x), y(y) {}
    
    Point& operator+=(Point p2) { x += p2.x; y += p2.y; return *this; }
    Point& operator-=(Point p2) { x -= p2.x; y -= p2.y; return *this; }
};

} // namespace

#endif /* POINT_H_ */