/*
 * Copyright 2013 C&D Elettronica. All rights reserved.
 *
 * This file is part of the FlashMat project.
 */

#include "Text.h"

#include <string.h>
#include "fmatdef.h"
#include "Fonts.h"

namespace FlashMat {

Text::Text() :
        charSpacing(1), //
        lineSpacing(2), //
        monospace(false), //
        overlay(false), //
        fontId(Fonts::NONE), //
        font(Fonts::getFont(fontId)) //
{
    // default: yellow
    color[R] = 255;
    color[G] = 255;
    color[B] = 0;
    
    // default: black
    bgColor[R] = 0;
    bgColor[G] = 0;
    bgColor[B] = 0;

    // also nullify the last byte, which can never be part of the string
    memset(str, 0, TEXT_BUFFER_SIZE + 1);

    topleft[X] = 0;
    topleft[Y] = 0;
}

Point Text::getStringPixelSize() const {
    coord_t w, h;
    getStringPixelSize(&w, &h);
    return Point(w, h);
}

void Text::getStringPixelSize(coord_t* width, coord_t* height) const {
    if(!height && !width)
        return;
        
    if(!font || str[0] == '\0') {
        if(width)
            *width = 0;
        if(height)
            *height = 0;
        return;
    }
    
    coord_t w = 0;
    coord_t h = 0;
    coord_t wRow = 0;
    
    // some caching
    const uint8_t fontH = font->height();
    const uint8_t charSpacing = this->charSpacing;
    const uint8_t lineSpacing = this->lineSpacing;
    const bool monospace = this->monospace;
    const uint8_t monoH = font->monoHeight();
    const uint8_t monoW = font->monoWidth();
    
    const char* s = str;
    char c;
    for(;;){
        c = *(s++);
        if (c == '\n' || c == '\0') {
            if(wRow > w){
                w = wRow;
            }
            wRow = 0;
            
            if (monospace) {
                // h += monoH;
                h += monoH + lineSpacing;
            } else {
                h += fontH + lineSpacing;
            }
            
            // END OF LOOP
            if(c == '\0')
                break;
        } else {
            const uint8_t cw = font->width(c);
            if (cw > 0) {
                if (monospace) {
                    // rowW += monoW;
                    wRow += monoW + charSpacing;
                } else {
                    wRow += cw + charSpacing;
                }
            }
        }
    }
    
    // we added one spacing too much, after the last character of each row (but only if the string wasn't empty!)
    // and after the last row: correct it (but only if the row/string wasn't empty, made only of invisible characters)
    if(height) { *height = (h ? h - lineSpacing : h); }
    if(width)  { *width  = (w ? w - charSpacing : w); }
}

} //namespace
