#ifndef PAIR_H
#define PAIR_H

namespace std {

    template<class T, class U>
    struct pair {
        typedef T first_type;
        typedef U second_type;

        T first;
        U second;

        pair(): first(), second() {}
        pair(const T& t, const U& u): first(t), second(u) {}

        template<class T2, class U2>
        pair(const pair<T2, U2>& other): first(other.first), second(other.second) {}
    };

    template<class T, class U>
    inline
    bool operator<(const pair<T, U>& p1, const pair<T, U>& p2) {
        return (p1.first < p2.first) || (!(p2.first < p1.first) && p1.second < p2.second);
    }

    template<class T, class U>
    inline
    bool operator==(const pair<T, U>& p1, const pair<T, U>& p2) {
        return p1.first == p2.first && p1.second == p2.second;
    }

    

    template<class T, class U>
    inline
    bool operator>(const pair<T, U>& p1, const pair<T, U>& p2) {
        return p2 < p1;
    }

    template<class T, class U>
    inline
    bool operator<=(const pair<T, U>& p1, const pair<T, U>& p2) {
        return !(p2 < p1);
    }

    template<class T, class U>
    inline
    bool operator>=(const pair<T, U>& p1, const pair<T, U>& p2) {
        return !(p1 < p2);
    }
        
    template<class T, class U>
    inline
    bool operator!=(const pair<T, U>& p1, const pair<T, U>& p2) {
        return !(p1 == p2);
    }

    template<class T, class U>
    inline
    pair<T, U> make_pair(T t, U u) {
        return pair<T, U>(t, u);
    }

} // namespace std

#endif /* PAIR_H */